import path from 'path';
import process from 'process';
import { defineConfig, loadEnv } from 'vite';
import vue from '@vitejs/plugin-vue';
import AutoImport from 'unplugin-auto-import/vite';
import Components from 'unplugin-vue-components/vite';
import { ElementPlusResolver, ArcoResolver, VantResolver } from 'unplugin-vue-components/resolvers';
import { visualizer } from 'rollup-plugin-visualizer';
import vueI18n from '@intlify/unplugin-vue-i18n/vite';
import { chunkSplitPlugin } from 'vite-plugin-chunk-split';
import imagemin from 'unplugin-imagemin/vite';
import DefineOptions from 'unplugin-vue-define-options/vite';
import { viteZip } from 'vite-plugin-zip-file';
import UnpluginGlob from 'unplugin-glob/vite';
import fs from 'fs-extra';
import portfinder from 'portfinder';

import { cliDir, appDir, srcDir, yicodeDir, cacheDir } from './config.js';
import { fnGetFileProtocolPath, fnImportModule } from './utils.js';

// TODO: 完成图标使用方案

export default defineConfig(async ({ command, mode }) => {
    let { yiviteConfig } = await fnImportModule(fnGetFileProtocolPath(path.resolve(yicodeDir, 'yivite.config.mjs')), {});
    let { dependencies = {} } = fs.readJsonSync(path.resolve(appDir, 'package.json'), { throws: false }) || {};

    let findPort = await portfinder.getPortPromise({ port: 8000, stopPort: 9000 });
    let rollupInput = {
        development: path.resolve(appDir, 'dev.html'),
        production: path.resolve(appDir, 'index.html')
    }[mode];

    // 每个项目依赖包进行分割
    let splitDependencies = {};
    let includeDeps = [];
    for (let prop in dependencies) {
        if (dependencies.hasOwnProperty(prop)) {
            splitDependencies[prop] = [prop];
            includeDeps.push(prop);
        }
    }
    let viteConfig = Object.assign(
        {
            plugins: [
                //
                vue({
                    reactivityTransform: true
                }),
                UnpluginGlob(),
                AutoImport({
                    include: [/\.[tj]sx?$/, /\.vue$/, /\.vue\?vue/, /\.md$/],
                    imports: ['vue'],
                    dirs: [
                        //
                        path.resolve(srcDir, 'components'),
                        path.resolve(srcDir, 'plugins')
                    ],
                    vueTemplate: true,
                    dts: '.yicode/auto-imports.d.ts',
                    resolvers: [ElementPlusResolver(), ArcoResolver()]
                }),
                Components({
                    directoryAsNamespace: true,
                    resolvers: [
                        //
                        ElementPlusResolver(),
                        ArcoResolver({
                            sideEffect: true
                        }),
                        VantResolver()
                    ]
                }),
                vueI18n({
                    include: path.resolve(srcDir, 'i18n/**')
                }),
                chunkSplitPlugin({
                    strategy: 'default',
                    customSplitting: Object.assign(splitDependencies, yiviteConfig?.chunkSplit || {})
                }),
                DefineOptions(),
                imagemin({
                    mode: 'sharp',
                    compress: {
                        jpg: {
                            quality: 80
                        },
                        jpeg: {
                            quality: 80
                        },
                        png: {
                            quality: 80
                        },
                        webp: {
                            quality: 80
                        }
                    },
                    cache: true
                }),
                viteZip({
                    folderPath: path.resolve(appDir, 'dist'),
                    outPath: cacheDir,
                    zipName: yiviteConfig?.viteZip?.zipName || 'dist.zip',
                    enabled: mode === 'production' ? true : false
                }),
                visualizer({
                    open: false,
                    brotliSize: true,
                    filename: '.cache/buildReport.html'
                })
            ],
            css: {
                preprocessorOptions: {
                    scss: {
                        additionalData: `@use "@/styles/variable.scss" as *;`
                    }
                }
            },
            resolve: {
                alias: [
                    {
                        find: '@',
                        replacement: path.resolve(srcDir)
                    }
                ]
            },
            optimizeDeps: includeDeps,
            root: appDir,
            base: './',
            envDir: path.resolve(srcDir, 'env'),
            build: {
                reportCompressedSize: false,
                chunkSizeWarningLimit: 1024,
                rollupOptions: {
                    plugins: [],
                    // 覆盖默认的 .html 入口
                    input: rollupInput,
                    output: {
                        // TODO: 进一步研究
                        // assetFileNames: ({ name }) => {
                        //     if (/\.(gif|jpe?g|png|svg)$/.test(name ?? '')) {
                        //         return 'assets/images/[name]-[hash][extname]';
                        //     }
                        //     if (/\.css$/.test(name ?? '')) {
                        //         return 'assets/css/[name]-[hash][extname]';
                        //     }
                        //     }
                        //     return 'assets/[name]-[hash][extname]';
                        // }
                    }
                }
            },
            server: {
                host: '127.0.0.1',
                port: findPort
            }
        },
        yiviteConfig?.viteConfig || {}
    );

    return viteConfig;
});
